@@ -1,21 +1,21 @@
 #include <stdio.h>
-#include <GToolKit/GToolKit.h>
-#include <Foundation/NSAutoreleasePool.h>
 #include <Foundation/NSArray.h>
+#include <Foundation/NSAutoreleasePool.h>
+#include <GToolKit/GToolKit.h>
 
 @interface GTKCListExample : GTKCList
-- (void)   select:sender row:(int) row column:(int) column;
-- (void) unselect:sender row:(int) row column:(int) column;
+- (void)   select:(id) sender row:(int) row column:(int) column;
+- (void) unselect:(id) sender row:(int) row column:(int) column;
 @end
 
 @implementation GTKCListExample
 
-- initWithColumns:(gint) columns
+- (id) initWithColumns:(gint) columns
 {
     return [self initWithTitles:columns titles:nil];
 }
 
-- initWithTitles:(gint) columns titles:(NSArray *) titles
+- (id) initWithTitles:(gint) columns titles:(NSArray *) titles
 {
     if (titles)
 	[super initWithTitles:columns titles:titles];
@@ -31,12 +31,12 @@
     return self;
 }
 
-- (void) select:sender row:(int) row column:(int) column
+- (void) select:(id) sender row:(int) row column:(int) column
 {
     fprintf(stderr, "select_row:   (%d,%d)\n", row, column);
 }
 
-- (void) unselect:sender row:(int) row column:(int) column
+- (void) unselect:(id) sender row:(int) row column:(int) column
 {
     fprintf(stderr, "unselect_row: (%d,%d)\n", row, column);
 }
@@ -47,10 +47,6 @@ int main (int argc, char *argv[])
     NSAutoreleasePool *pool = [NSAutoreleasePool new];
     GTKWindow *window;
     GTKCListExample *clist;
-#ifdef LIB_FOUNDATION_LIBRARY
-    extern char **environ;
-    [NSProcessInfo initializeWithArguments:argv count:argc environment:environ];
-#endif
 
     [[GTKApplication alloc] initWithArgc:&argc argv:&argv];
 
