@@ -19,7 +19,7 @@
 
 @implementation GLess
 
-- init
+- (id) init
 {
     GTKWindow *window = [GTKWindow windowWithType:GTK_WINDOW_TOPLEVEL];
     GTKScrolledWindow *scroll =
@@ -74,18 +74,14 @@
 int main (int argc, char *argv[])
 {
     NSAutoreleasePool *pool = [NSAutoreleasePool new];
-#ifdef LIB_FOUNDATION_LIBRARY
-    extern char **environ;
-    [NSProcessInfo initializeWithArguments:argv count:argc environment:environ];
-#endif
 
     /* initialize glib thread support */
     g_thread_init(NULL);
 
-    if (argc > 1 && !freopen(argv[1], "r", stdin)) perror(argv[1]), exit(1);
-
     [[GTKApplication alloc] initWithArgc:&argc argv:&argv];
 
+    if (argc > 1 && !freopen(argv[1], "r", stdin)) perror(argv[1]), exit(1);
+
     gdk_threads_enter();
     [[GLess new] autorelease];
 
