@@ -1,7 +1,8 @@
-#include <GToolKit/GToolKit.h>
-#include <Foundation/NSAutoreleasePool.h>
 #include <Foundation/NSArray.h>
+#include <Foundation/NSAutoreleasePool.h>
+#include <Foundation/NSProcessInfo.h>
 #include <Foundation/NSUtilities.h>
+#include <GToolKit/GToolKit.h>
 
 int main (int argc, char *argv[])
 {
@@ -10,13 +11,12 @@ int main (int argc, char *argv[])
     GTKLabel *label;
     GTKButton *button;
     GTKVBox *box;
-#ifdef LIB_FOUNDATION_LIBRARY
-    extern char **environ;
-    [NSProcessInfo initializeWithArguments:argv count:argc environment:environ];
-#endif
 
     [[GTKApplication alloc] initWithArgc:&argc argv:&argv];
 
+    NSLog([[[NSProcessInfo processInfo] arguments] description]);
+    [GTOOLKIT_OBJECT(gtk_button_new()) release];
+
     window = [GTKWindow windowWithType:GTK_WINDOW_TOPLEVEL];
     [window setBorderWidth:10];
 
