@@ -1,20 +1,20 @@
 /*
  * "p" - A very simple Project Manager using the GIMP Toolkit
- * Copyright (c) 1998  Elmar Ludwig - Universitaet Osnabrueck
+ * Copyright (c) 1998, 1999, 2000  Elmar Ludwig
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  */
 #ifndef MANAGER_H
@@ -48,22 +48,22 @@
     NSString *filename;
 }
 
-- (void) info:sender;
-- (void) select:sender;
-- (void) selectFile:sender;
-- (void) add:sender;
+- (void) info:(id) sender;
+- (void) select:(id) sender;
+- (void) selectFile:(id) sender;
+- (void) add:(id) sender;
 - (void) addFile:(GTKListItem *) group;
-- (void) ok:sender;
-- (void) cancel:sender;
-- (void) remove:sender;
-- (void) open:sender;
-- (void) launch:sender;
-- (void) debug:sender;
-- (void) attrib:sender;
-- (void) build:sender;
+- (void) ok:(id) sender;
+- (void) cancel:(id) sender;
+- (void) remove:(id) sender;
+- (void) open:(id) sender;
+- (void) launch:(id) sender;
+- (void) debug:(id) sender;
+- (void) attrib:(id) sender;
+- (void) build:(id) sender;
 - (void) loadFromFile:(NSString *) filename;
-- (void) save:sender;
+- (void) save:(id) sender;
 - (void) saveToFile:(NSString *) filename;
 @end
 
-#endif
+#endif /* MANAGER_H */
