@@ -1,20 +1,20 @@
 /*
  * "p" - A very simple Project Manager using the GIMP Toolkit
- * Copyright (c) 1998  Elmar Ludwig - Universitaet Osnabrueck
+ * Copyright (c) 1998, 1999, 2000  Elmar Ludwig
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  */
 #ifdef HAVE_CONFIG_H
@@ -31,16 +31,17 @@
 extern char *getcwd ();
 #endif
 
-#include <GToolKit/GToolKit.h>
 #include <Foundation/NSArray.h>
 #include <Foundation/NSDictionary.h>
+#include <Foundation/NSPathUtilities.h>
+#include <GToolKit/GToolKit.h>
 #include "Manager.h"
 
 #define KEY(listitem)	[[listitem child] stringValue]
 
 @implementation Manager
 
-- init
+- (id) init
 {
     GTKWindow *window = [GTKWindow windowWithType:GTK_WINDOW_TOPLEVEL];
     GTKVBox *vbox = [GTKVBox vBoxWithHomogeneous:NO spacing:0];
@@ -64,8 +65,8 @@ extern char *getcwd ();
     GTKAction *select =
 	[GTKAction actionWithTarget:self selector:@selector(select:)];
     GTKTooltips *tips = [GTKTooltips tooltips];
-    int index;
     char buffer[1024];
+    int index;
 
     static GTKItemFactoryEntry menus[] =
     {
@@ -77,7 +78,7 @@ extern char *getcwd ();
 
 	{{ "/_File", NULL, 0, 0, "<Branch>" }},
 	{{ "/File/_Add...",		"<control>A" },	@selector(add:) },
-	{{ "/File/sep2", NULL, 0, 0, "<Separator>" }},
+	{{ "/File/sep1", NULL, 0, 0, "<Separator>" }},
 	{{ "/File/_Open",		"<control>O" },	@selector(open:) },
 	{{ "/File/_Remove",		"<control>R" },	@selector(remove:) },
 
@@ -155,7 +156,6 @@ extern char *getcwd ();
     [window setUsize:320 height:220];
     [window setTitle:[NSString stringWithFormat:@" %@ - %@ ", program,
 				[path stringByAbbreviatingWithTildeInPath]]];
-    [menubar show];
     [window show];
 
     files = [[NSMutableDictionary alloc] initWithCapacity:8];
@@ -181,14 +181,14 @@ extern char *getcwd ();
     [super dealloc];
 }
 
-- (void) info:sender
+- (void) info:(id) sender
 {
     GTKRunMessageBox(@"Info Panel", @"This is an example of a very simple\n"
 		     @"Project Manager using the GIMP Toolkit.",
 		     @"Continue", nil, nil);
 }
 
-- (void) select:sender
+- (void) select:(id) sender
 {
     [list2 removeItems:[list2 children]];
     [list2 appendItems:[files objectForKey:KEY(sender)]];
@@ -196,13 +196,13 @@ extern char *getcwd ();
 	[self addFile:sender];
 }
 
-- (void) selectFile:sender
+- (void) selectFile:(id) sender
 {
     if ([GTKApp getCurrentEvent]->type == GDK_2BUTTON_PRESS)
 	[self open:sender];
 }
 
-- (void) add:sender
+- (void) add:(id) sender
 {
     NSArray *sel = [list1 selection];
 
@@ -240,17 +240,17 @@ extern char *getcwd ();
     [filesel release];
 }
 
-- (void) ok:sender
+- (void) ok:(id) sender
 {
     [GTKApp stopModalWithCode:YES];
 }
 
-- (void) cancel:sender
+- (void) cancel:(id) sender
 {
     [GTKApp stopModalWithCode:NO];
 }
 
-- (void) remove:sender
+- (void) remove:(id) sender
 {
     NSArray *sel1 = [list1 selection];
     NSArray *sel2 = [list2 selection];
@@ -266,7 +266,7 @@ extern char *getcwd ();
     [self save:self];
 }
 
-- (void) open:sender
+- (void) open:(id) sender
 {
     NSArray *sel2 = [list2 selection];
     NSString *command;
@@ -277,7 +277,7 @@ extern char *getcwd ();
     system([command cString]);
 }
 
-- (void) launch:sender
+- (void) launch:(id) sender
 {
     NSString *command = [NSString stringWithFormat:@"%@ '%@' &",
 				  LAUNCH_CMD, program];
@@ -285,7 +285,7 @@ extern char *getcwd ();
     system([command cString]);
 }
 
-- (void) debug:sender
+- (void) debug:(id) sender
 {
     NSString *command = [NSString stringWithFormat:@"%@ '%@' &",
 				  DEBUG_CMD, program];
@@ -293,7 +293,7 @@ extern char *getcwd ();
     system([command cString]);
 }
 
-- (void) attrib:sender
+- (void) attrib:(id) sender
 {
     NSString *command = [NSString stringWithFormat:@"%@ '%@' &",
 				  EDIT_CMD, @"Makefile"];
@@ -301,7 +301,7 @@ extern char *getcwd ();
     system([command cString]);
 }
 
-- (void) build:sender
+- (void) build:(id) sender
 {
     static GdkFont *font;
     GTKWindow *output = [GTKWindow windowWithType:GTK_WINDOW_TOPLEVEL];
@@ -382,7 +382,7 @@ extern char *getcwd ();
     fclose(file);
 }
 
-- (void) save:sender
+- (void) save:(id) sender
 {
     if (!filename)
 	filename = [[program stringByAppendingString:@".project"] retain];
