@@ -5,17 +5,17 @@
 #include <GToolKit/GToolKit.h>
 
 @interface Foo : NSObject <GTKApplicationDelegate, GTKWindowDelegate>
-- (void) newWindow:sender;
-- (void) dumpInfo:sender;
+- (void) newWindow:(id) sender;
+- (void) dumpInfo:(id) sender;
 - (void) printNotification:(NSNotification *) event;
 #ifdef DEBUG
-- (BOOL) print:sender event:(GdkEvent *) event;
+- (BOOL) print:(id) sender event:(GdkEvent *) event;
 #endif
 @end
 
 @implementation Foo
 
-- (void) newWindow:sender
+- (void) newWindow:(id) sender
 {
     GTKWindow *window = [GTKWindow windowWithType:GTK_WINDOW_TOPLEVEL];
     GTKBox *box = [GTKVBox vBoxWithHomogeneous:YES spacing:0];
@@ -34,7 +34,7 @@
     [window show];
 }
 
-- (void) dumpInfo:sender
+- (void) dumpInfo:(id) sender
 {
     fprintf(stderr, "isRunning:\t%d\n", [GTKApp isRunning]);
     fprintf(stderr, "isActive:\t%d\n", [GTKApp isActive]);
@@ -48,7 +48,7 @@
 }
 
 #ifdef DEBUG
-- (BOOL) print:sender event:(GdkEvent *) event
+- (BOOL) print:(id) sender event:(GdkEvent *) event
 {
     static const char *event_names[] =
     {
@@ -153,10 +153,6 @@ int main (int argc, char *argv[])
 {
     NSAutoreleasePool *pool = [NSAutoreleasePool new];
     Foo *foo = [[Foo new] autorelease];
-#ifdef LIB_FOUNDATION_LIBRARY
-    extern char **environ;
-    [NSProcessInfo initializeWithArguments:argv count:argc environment:environ];
-#endif
 
 #ifdef DEBUG
     [[NSNotificationCenter defaultCenter] addObserver:foo
