@@ -9,33 +9,29 @@
     GTKDialog *info;
     GTKText *text;
 }
-- (void) open:sender;
-- (void) clicked:sender data:(NSString *) data;
+- (void) open:(id) sender;
+- (void) clicked:(id) sender data:(NSString *) data;
 @end
 
 @implementation Manager
 
-- init
+- (id) init
 {
-    NSDictionary *name_table = [NSDictionary dictionaryWithObjectsAndKeys:
-				    self, @"owner", GTKApp, @"gtkapp", nil];
-    GTKGladeXML *tree;
+    NSDictionary *table = [NSDictionary dictionaryWithObjectsAndKeys:
+				self, @"owner", GTKApp, @"gtkapp", nil];
 
     [super init];
 
-    tree = [GTKGladeXML gladeXMLFromFile:@"xml.glade" rootObject:nil
-			nameTable:name_table];
-    if (tree == nil) g_error("loading user interface failed");
-
-    [tree resolveReferences:@"window", &window, @"file_dialog", &filesel,
-			    @"info_dialog", &info, @"text_area", &text, nil];
+    [[GTKGladeXML gladeXMLFromFile:@"xml.glade" rootObject:nil nameTable:table]
+	resolveReferences:@"window", &window, @"file_dialog", &filesel,
+			  @"info_dialog", &info, @"text_area", &text, nil];
     [window autorelease];
     [filesel autorelease];
     [info autorelease];
     return self;
 }
 
-- (void) open:sender
+- (void) open:(id) sender
 {
     if ([GTKApp runModalForWindow:filesel])
     {
@@ -48,7 +44,7 @@
     [filesel hide];
 }
 
-- (void) clicked:sender data:(NSString *) data
+- (void) clicked:(id) sender data:(NSString *) data
 {
     [GTKApp stopModalWithCode:[data isEqualToString:@"ok"]];
 }
@@ -57,10 +53,6 @@
 int main (int argc, char *argv[])
 {
     NSAutoreleasePool *pool = [NSAutoreleasePool new];
-#ifdef LIB_FOUNDATION_LIBRARY
-    extern char **environ;
-    [NSProcessInfo initializeWithArguments:argv count:argc environment:environ];
-#endif
 
     [[GTKApplication alloc] initWithArgc:&argc argv:&argv];
     [[Manager new] autorelease];
