@@ -1,20 +1,20 @@
 /*
  * GToolKit - Objective-C interface to the GIMP Toolkit
- * Copyright (c) 2000  Elmar Ludwig - Universitaet Osnabrueck
+ * Copyright (c) 1998, 1999, 2000  Elmar Ludwig
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  */
 #ifdef HAVE_CONFIG_H
@@ -23,17 +23,21 @@
 
 #include <ctype.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
 #include <Foundation/NSArray.h>
 #include <Foundation/NSAutoreleasePool.h>
+#include <Foundation/NSException.h>
 #include <Foundation/NSLock.h>
-#include <Foundation/NSMapTable.h>
 #include <Foundation/NSObjCRuntime.h>
 #include <Foundation/NSThread.h>
-#include <GToolKit/GToolKit.h>
+#include <GToolKit/GTK.h>
+#include <GToolKit/GTKAction.h>
+#include <GToolKit/GTKObject.h>
+#include <gtk/gtksignal.h>
 #ifdef HAVE_LIBGLADE
-#include <glade/glade.h>
-#endif
+#include <glade/glade-xml.h>
+#endif /* HAVE_LIBGLADE */
 
 /*
  * Objects of this class can autorelease standard C pointers.
@@ -41,51 +45,91 @@
 @interface __GAutorelease : NSObject
 {
     gpointer object;
-    void (*func)(gpointer);
+    GFreeFunc function;
 }
-+ (gpointer) release:(gpointer) object with:(void (*)(gpointer)) func;
++ (gpointer) release:(gpointer) object with:(GFreeFunc) function;
 @end
 
 @implementation __GAutorelease
 
-+ (gpointer) release:(gpointer) _object with:(void (*)(gpointer)) _func
++ (gpointer) release:(gpointer) _object with:(GFreeFunc) _function
 {
     __GAutorelease *obj = [[self new] autorelease];
 
     obj->object = _object;
-    obj->func = _func;
+    obj->function = _function;
     return _object;
 }
 
 - (void) dealloc
 {
 #ifdef DEBUG
-    fprintf(stderr, "(__GAutorelease dealloc) object = %p\n", object);
+    fprintf(stderr, "[__GAutorelease dealloc] object = %p\n", object);
 #endif
-    func(object);
+    function(object);
     [super dealloc];
 }
 @end
 
 /*
- * The String_to_NSString() macro does not allow its argument to be NULL,
- * this function does. To call it, you have to #undef String_to_NSString.
+ * Convert UTF-8 C string to NSString object.
  */
-NSString *(String_to_NSString) (const char *string)
+NSString *gtoolkit_objc_string (const char *string)
 {
-    return string ? String_to_NSString(string) : nil;	// macro
+    // GTK+ 1.2 does not support UTF-8, so use ASCII instead
+    return string ? [NSString stringWithCString:string] : nil;
 }
 
 /*
- * Some general type conversion functions for NSArray.
+ * Convert NSString object to UTF-8 C string.
  */
-static void add_to_array (gpointer data, NSMutableArray *array)
+const char *gtoolkit_utf8_string (NSString *string)
 {
-    // XXX: this will only work if data is some kind of GtkObject
-    [array addObject:GTK_GET_OBJECT(data)];
+    // GTK+ 1.2 does not support UTF-8, so use ASCII instead
+    return [string cString];
 }
 
-NSArray *GList_to_NSArray (GList *glist)
+/*
+ * Convert NSArray to GList.
+ */
+GList *gtoolkit_array_to_list (NSArray *array)
+{
+    int index = [array count];
+    GList *list = NULL;
+
+    while (index--)
+    {
+	id obj = [array objectAtIndex:index];
+	const void *data;
+
+	data = [obj isKindOfClass:[GTK class]] ? [obj gtk]
+	     : [obj isKindOfClass:[NSString class]] ? gtoolkit_utf8_string(obj)
+//	     : [obj isKindOfClass:[NSData class]] ? [obj bytes]
+	     : [obj isKindOfClass:[NSArray class]] ? gtoolkit_array_to_list(obj)
+	     : (const void *) gtoolkit_utf8_string([obj description]);
+	list = g_list_prepend(list, (gpointer) data);
+    }
+
+    // Hmm, gtk_list_insert_items() implicitly takes ownership of its second
+    // argument, so we must not free this list here, which leads to a small
+    // memory leak in all other cases...
+#ifdef GLIST_AUTORELEASE
+    return [__GAutorelease release:list with:(GFreeFunc) g_list_free];
+#else
+    return list;
+#endif /* GLIST_AUTORELEASE */
+}
+
+static void add_to_array (gpointer data, NSMutableArray *array)
+{
+    // this will only work if data is some kind of GtkObject
+    [array addObject:GTOOLKIT_OBJECT(data)];
+}
+
+/*
+ * Convert GList of GTK+ objects to NSArray.
+ */
+NSArray *gtoolkit_list_to_array (GList *glist)
 {
     NSMutableArray *array;
 
@@ -95,105 +139,139 @@ NSArray *GList_to_NSArray (GList *glist)
     return array;
 }
 
-GList *NSArray_to_GList (NSArray *array)
+/*
+ * Convert NSArray of NSString objects to NULL-terminated vector of strings.
+ */
+const char **gtoolkit_array_to_strvec (NSArray *array)
 {
-    GList *list = NULL;
     int index, count = [array count];
+    const char **vector;
 
-    for (index = 0; index < count; ++index)
-    {
-	id obj = [array objectAtIndex:index];
-	const void *data;
+    if (array == nil) return NULL;
 
-	data = [obj isKindOfClass:[GTK class]] ? [obj gtk]
-	     : [obj isKindOfClass:[NSString class]] ? NSString_to_String(obj)
-//	     : [obj isKindOfClass:[NSData class]] ? [obj bytes]
-	     : [obj isKindOfClass:[NSArray class]] ? NSArray_to_GList(obj)
-	     : (gpointer) NSString_to_String([obj description]);
+    vector = g_new(const char *, count + 1);
+    vector[count] = NULL;
+    for (index = 0; index != count; ++index)
+	vector[index] = gtoolkit_utf8_string([array objectAtIndex:index]);
 
-	list = g_list_append(list, (gpointer) data);
-    }
-
-    // XXX: check if this is still necessary
-#ifdef GLIST_AUTORELEASE
-    return [__GAutorelease release:list with:(void (*)(gpointer)) g_list_free];
-#else
-    return list;
-#endif /* GLIST_AUTORELEASE */
+    return [__GAutorelease release:vector with:g_free];
 }
 
-NSArray *StrVec_to_NSArray (const char **strvec)
+/*
+ * Convert NULL-terminated vector of strings to NSArray of NSString objects.
+ */
+NSArray *gtoolkit_strvec_to_array (const char **vector)
 {
     NSMutableArray *array = nil;
 
-    if (strvec)
+    if (vector)
     {
 	array = [NSMutableArray arrayWithCapacity:4];
-	while (*strvec) [array addObject:String_to_NSString(*strvec++)];
+	while (*vector) [array addObject:gtoolkit_objc_string(*vector++)];
     }
 
     return array;
 }
 
-const char **NSArray_to_StrVec (NSArray *array)
+BOOL gtoolkit_debug;			// enable limited debug output
+static NSLock *global_lock;		// lock for all global variables
+static GHashTable *object_table;	// map gpointer to object
+
+/*
+ * Initialize some internal (non-objc) variables and data structures.
+ * The GTKApplication class will call this function on startup.
+ */
+void gtoolkit_init (void)
 {
-    int index, count = [array count];
-    const char **vec;
+#ifndef DEBUG
+    if (getenv("GTOOLKIT_DEBUG"))
+#endif
+	gtoolkit_debug = YES;
 
-    if (array == nil) return NULL;
-
-    vec = g_new(const char *, count + 1);
-    vec[count] = NULL;
-    for (index = 0; index < count; ++index)
-	vec[index] = NSString_to_String([array objectAtIndex:index]);
-
-    return [__GAutorelease release:vec with:g_free];
+    object_table = g_hash_table_new(g_direct_hash, g_direct_equal);
 }
 
-extern BOOL gtoolkit_debug;		// enable limited debug output
-static NSLock *global_lock;		// lock for all global variables
-static NSMapTable *object_table;	// map gpointer to object
-
 @implementation GTK
 
+/*
+ * Try to find the correct Objective-C class name for this object.
+ */
+static NSString *gtoolkit_class (GtkObject *gtk)
+{
+    NSString *result;
+    const char *name;
+    char *string;
+    int pos = 0, copy = 0, uc = 0;		// uc: upper case
+
+#ifdef HAVE_LIBGLADE
+    if ((name = glade_get_widget_name((GtkWidget *) gtk)) &&
+	(string = strstr(name, "::")))
+	// TODO: check that this class matches the actual GTK+ class
+	return [NSString stringWithCString:name length:string - name];
+#endif /* HAVE_LIBGLADE */
+
+    // guess Objective-C class name from the GTK+ class name
+    name = gtk_type_name(GTK_OBJECT_TYPE(gtk));
+    string = g_new(char, strlen(name) + 2);	// + '_' and '\0'
+
+    if (islower(name[0])) uc = 1;		// gtkFoo -> GtkFoo
+    else if (isupper(name[0]))
+    {
+	while (islower(name[++pos]));
+
+	if (pos > 3) copy = pos;		// GnomeFoo -> Gnome_Foo
+	else if (pos > 1) uc = pos;		// GtkFoo -> GTKFoo
+	else
+	{
+	    while (name[pos] && !islower(name[pos])) ++pos;
+
+	    if (name[pos]) copy = pos - 1;	// GTKFoo -> GTK_Foo
+	}
+    }
+
+    strncpy(string, name, copy);
+    for (pos = copy; pos < uc; ++pos) string[pos] = toupper(name[pos]);
+    if (!uc) string[pos] = '_';
+    strcpy(string + pos + !uc, name + pos);
+
+#ifdef DEBUG
+    fprintf(stderr, "(gtoolkit_class) %s -> %s\n", name, string);
+#endif
+    result = [NSString stringWithCString:string];
+    g_free(string);
+    return result;
+}
+
+/* Backwards compatibility function (TODO: obsolete) */
+id (Gtk_to_Object) (gpointer gtk, const char *class)
+{
+    return Gtk_to_Object(gtk, class);
+}
+
 /*
  * Find Objective-C object for given gpointer, create one if necessary.
  * CLASS is the object's type name (use NULL to autodetect type name).
  */
-id Gtk_to_Object (gpointer gtk, const char *class)
+id gtoolkit_object (gpointer gtk, const char *class)
 {
     id result;
 
     if (gtk == NULL) return nil;
 
     if (global_lock) [global_lock lock];
-    result = NSMapGet(object_table, gtk);
+    result = g_hash_table_lookup(object_table, gtk);
     if (global_lock) [global_lock unlock];
 
     if (result == nil)
     {
-	NSString *name = [NSString alloc];
-	char *text;
+	// class names are limited to ASCII characters
+	NSString *objc_class =
+	    class ? [NSString stringWithCString:class] : gtoolkit_class(gtk);
 
-	if (class) name = [name initWithCString:class];
-#ifdef HAVE_LIBGLADE
-	else if ((class = glade_get_widget_name(gtk)) &&
-		 (text = strstr(class, "::")))
-	    // TODO: check that this class matches the actual gtk class
-	    name = [name initWithCString:class length:text - class];
-#endif
-	else
-	{
-	    text = g_strdup(gtk_type_name(GTK_OBJECT_TYPE(gtk)));
-	    if (text[0]) text[1] = toupper(text[1]);	// GtkFoo -> GTKFoo
-	    if (text[1]) text[2] = toupper(text[2]);
-	    name = [name initWithCString:text];
-	    g_free(text);
-	}
+	if ((result = [NSClassFromString(objc_class) alloc]) == nil)
+	    [NSException raise:NSGenericException
+		format:@"cannot find objc class `%@'", objc_class];
 
-	if ((result = [NSClassFromString(name) alloc]) == nil)
-	    g_error("cannot find objc class `%s'", NSString_to_String(name));
-	[name release];
 	[result setTag:-1];			// disable "show by default"
 	result = [result initWithGtk:gtk];
     }
@@ -202,9 +280,9 @@ id Gtk_to_Object (gpointer gtk, const char *class)
 }
 
 /*
- * Notify the GTK runtime system that resource locking is necessary.
- * This method is intended only for use by the @GTKApplication class
- * and should not be called from user code.
+ * Notify the GToolKit runtime system that resource locking is necessary.
+ * This method is intended only for use by the @GTKApplication class and
+ * should never be called from user code.
  */
 + (void) taskNowMultiThreaded:(NSNotification *) event
 {
@@ -223,39 +301,38 @@ id Gtk_to_Object (gpointer gtk, const char *class)
 /*
  * Compare two objects for equality.
  */
-- (BOOL) isEqual:anObject
+- (BOOL) isEqual:(id) anObject
 {
-    if (![anObject isKindOfClass:[GTK class]]) return NO;
+    if ([anObject isKindOfClass:[GTK class]] == NO) return NO;
     return gtk == ((GTK *) anObject)->gtk;
 }
 #endif
 
- - init
+ - (id) init
 {
-    g_error("objects of type `%s' cannot be created with +new",
-	    NSString_to_String([isa description]));
+    [NSException raise :NSInvalidArgumentException	// avoid bug in objcdoc
+	format:@"objects of type `%@' cannot be created with +new", isa];
     return nil;
 }
 
 /*
- * Initialize a new GTK object for the given gtk-object pointer /gtk/,
- * which must not be |NULL|.<br>
- * This is an internal method and should never be called from user code.
+ * Initialize a new GTK object for the given gtk-object pointer /gtk/, which
+ * must not be |NULL|. This is an internal method and should generally not
+ * be called from user code, unless you really know what you are doing.<p>
+ * Use the |GTOOLKIT_OBJECT| macro (or the |gtoolkit_object| function) to
+ * create an Objective-C object from an existing gtk-object pointer.
  */
-- initWithGtk:(gpointer) _gtk
+- (id) initWithGtk:(gpointer) _gtk
 {
     [super init];
 
     [global_lock lock];
-    if (object_table == NULL)
-	object_table = NSCreateMapTable(NSNonOwnedPointerMapKeyCallBacks,
-					NSNonOwnedPointerMapValueCallBacks, 64);
-    NSMapInsert(object_table, gtk = _gtk, self);
+    g_hash_table_insert(object_table, *(gpointer *) &gtk = _gtk, self);
     [global_lock unlock];
 
     if (gtoolkit_debug)
 	fprintf(stderr, "[%s init] at %p: gtk = %p\n",
-	NSString_to_String([isa description]), self, gtk);
+		class_get_class_name(isa), self, gtk);	// GNU runtime
 
     return self;
 }
@@ -264,10 +341,10 @@ id Gtk_to_Object (gpointer gtk, const char *class)
 {
     if (gtoolkit_debug)
 	fprintf(stderr, "[%s dealloc] at %p: gtk = %p\n",
-	NSString_to_String([isa description]), self, gtk);
+		class_get_class_name(isa), self, gtk);	// GNU runtime
 
     [global_lock lock];
-    NSMapRemove(object_table, gtk);
+    if (gtk) g_hash_table_remove(object_table, gtk);
     [global_lock unlock];
     [actions release];
     [super dealloc];
@@ -284,16 +361,15 @@ id Gtk_to_Object (gpointer gtk, const char *class)
 #endif
 
 /*
- * Return the internal gtk-object pointer. This may be generally useful to
+ * Return the internal gtk-object pointer. This may sometimes be useful to
  * access components of the underlying gtk object that do not have access
- * methods. The macro call |GTK_GET_OBJECT(|/gtk_pointer/|)| can perform the
- * reverse operation. Example:
+ * methods. The macro call |GTOOLKIT_OBJECT(|/gtk_pointer/|)| can perform
+ * the reverse operation. Example:
  * <pre>
- * GTKStyle *style = [button getStyle];
- * GtkStyle *gtk_style = [style gtk];
+ * GTKButton *button = [GTKButton buttonWithLabel:label];
+ * GtkButton *gtk_button = [button gtk];
  *
- * gdk_color_parse("yellow", \&gtk_style-\>bg[GTK_STATE_NORMAL]);
- * gdk_color_alloc(colormap, \&gtk_style-\>bg[GTK_STATE_NORMAL]);</pre>
+ * GTK_WIDGET_SET_FLAGS(gtk_button, GTK_CAN_DEFAULT);</pre>
  */
 - (gpointer) gtk
 {
@@ -327,21 +403,22 @@ id Gtk_to_Object (gpointer gtk, const char *class)
 + (void) addUserSignal:(NSString *) signal flags:(GtkSignalRunType) flags
 {
     gtk_object_class_user_signal_new(gtk_type_class([self getType]),
-				     NSString_to_String(signal), flags,
+				     gtoolkit_utf8_string(signal), flags,
 				     gtk_signal_default_marshaller,
 				     GTK_TYPE_NONE, 0);
 }
 
 /*
- * Tell the receiver to emit the given /signal/, i.e. invoke all actions
- * connected to this signal. The /sender/ parameter is ignored.
+ * Tell the receiver to emit the given /signal/ (without parameters), i.e.
+ * invoke all actions connected to this signal. The /sender/ parameter is
+ * ignored.
  * @see -stop:signal:
  */
-- (void) emit:sender signal:(NSString *) signal
+- (void) emit:(id) sender signal:(NSString *) signal
 {
     union { long l; double d; void *p; } result;
 
-    gtk_signal_emit_by_name(gtk, NSString_to_String(signal), &result);
+    gtk_signal_emit_by_name(gtk, gtoolkit_utf8_string(signal), &result);
 }
 
 /*
@@ -350,9 +427,9 @@ id Gtk_to_Object (gpointer gtk, const char *class)
  * Toolkit documentation for details. The /sender/ parameter is ignored.
  * @see -emit:signal:
  */
-- (void) stop:sender signal:(NSString *) signal
+- (void) stop:(id) sender signal:(NSString *) signal
 {
-    gtk_signal_emit_stop_by_name(gtk, NSString_to_String(signal));
+    gtk_signal_emit_stop_by_name(gtk, gtoolkit_utf8_string(signal));
 }
 
 /*
@@ -365,10 +442,9 @@ static void forward_signal (gpointer gtk, GTKAction *action, guint n_args,
 
 #ifdef DEBUG
     fprintf(stderr, "(forward_signal init) gtk = %p, action [%p %s %p]\n", gtk,
-	    [action target], NSSelector_to_String([action selector]),
-	    [action data]);
+	    [action target], sel_get_name([action selector]), [action data]);
 #endif
-    [action performWithSender:GTK_GET_OBJECT(gtk) args:args count:n_args];
+    [action performWithSender:GTOOLKIT_OBJECT(gtk) args:args count:n_args];
     [pool release];
 #ifdef DEBUG
     fprintf(stderr, "(forward_signal exit) gtk = %p\n", gtk);
@@ -382,9 +458,9 @@ static void forward_signal (gpointer gtk, GTKAction *action, guint n_args,
  * pass itself as the sending object when the given /signal/ is emitted.
  * Example:
  * <pre>
- * - (void) foo:sender
+ * - (void) foo:(id) sender
  * ...
- * - (void) bar:sender data:(myType *) data
+ * - (void) bar:(id) sender data:(myType *) data
  * ...
  *
  * [window connectSignal:@"destroy" withAction:
@@ -396,7 +472,7 @@ static void forward_signal (gpointer gtk, GTKAction *action, guint n_args,
  */
 - (void) connectSignal:(NSString *) signal withAction:(GTKAction *) action
 {
-    gtk_signal_connect_full(gtk, NSString_to_String(signal), NULL,
+    gtk_signal_connect_full(gtk, gtoolkit_utf8_string(signal), NULL,
 		    (GtkCallbackMarshal) forward_signal, action, NULL, NO, NO);
     if (actions == nil) actions = [NSMutableArray new];
     [actions addObject:action];
@@ -411,7 +487,7 @@ static void forward_signal (gpointer gtk, GTKAction *action, guint n_args,
  * Use the @-connectSignal:withAction: method if you need a reference
  * to the GTKAction object (to block/unblock\/disconnect it later).
  */
-- (void) connectSignal:(NSString *) signal withTarget:target sel:(SEL) sel
+- (void) connectSignal:(NSString *) signal withTarget:(id) target sel:(SEL) sel
 {
     [self connectSignal:signal withAction:
 	  [GTKAction actionWithTarget:target selector:sel]];
@@ -426,7 +502,7 @@ static void forward_signal (gpointer gtk, GTKAction *action, guint n_args,
  * Use the @-connectSignal:withAction: method if you need a reference
  * to the GTKAction object (to block/unblock\/disconnect it later).
  */
-- (void) connectSignal:(NSString *) signal withTarget:target sel:(SEL) sel
+- (void) connectSignal:(NSString *) signal withTarget:(id) target sel:(SEL) sel
 	 data:(const void *) data
 {
     [self connectSignal:signal withAction:
@@ -442,7 +518,7 @@ static void forward_signal (gpointer gtk, GTKAction *action, guint n_args,
  */
 - (void) connectSignal:(NSString *) signal withActionAfter:(GTKAction *) action
 {
-    gtk_signal_connect_full(gtk, NSString_to_String(signal), NULL,
+    gtk_signal_connect_full(gtk, gtoolkit_utf8_string(signal), NULL,
 		    (GtkCallbackMarshal) forward_signal, action, NULL, NO, YES);
     if (actions == nil) actions = [NSMutableArray new];
     [actions addObject:action];
