@@ -1,28 +1,28 @@
 /*
  * GToolKit - Objective-C interface to the GIMP Toolkit
- * Copyright (c) 2000  Elmar Ludwig - Universitaet Osnabrueck
+ * Copyright (c) 1998, 1999, 2000  Elmar Ludwig
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  */
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
 
-#include <GToolKit/GTKAction.h>
 #include <GToolKit/GTK.h>
+#include <GToolKit/GTKAction.h>
 
 @implementation GTKAction
 
@@ -30,7 +30,7 @@
  * Create an autoreleased GTKAction that will send /selector/ to /target/.
  * @see -performWithSender:args:count:
  */
-+ actionWithTarget:_target selector:(SEL) _selector
++ (id) actionWithTarget:(id) _target selector:(SEL) _selector
 {
     return [[[self alloc] initWithTarget:_target selector:_selector]
 	    autorelease];
@@ -39,19 +39,20 @@
 /*
  * Create an autoreleased GTKAction that will send /selector/ to /target/.
  * /data/ will be passed as an additional argument to the method.
- * Note that /data/ is not retained by default.
+ * Note that /data/ is not retained by default (if it is an object).
  */
-+ actionWithTarget:_target selector:(SEL) _selector data:(const void *) _data
++ (id) actionWithTarget:(id) _target selector:(SEL) _selector
+       data:(const void *) _data
 {
     return [[[self alloc] initWithTarget:_target selector:_selector data:_data]
 	    autorelease];
 }
 
 /*
- * Initialize a new GTKAction without /selector/ and /target/.
+ * Initialize a new GTKAction without /target/ and /selector/.
  * @see -setTarget:selector:
  */
-- init
+- (id) init
 {
     return [self initWithTarget:nil selector:0];
 }
@@ -60,7 +61,7 @@
  * Initialize a new GTKAction that will send /selector/ to /target/.
  * @see -performWithSender:args:count:
  */
-- initWithTarget:_target selector:(SEL) _selector
+- (id) initWithTarget:(id) _target selector:(SEL) _selector
 {
     return [self initWithTarget:_target selector:_selector data:nil];
 }
@@ -68,9 +69,10 @@
 /*
  * Initialize a new GTKAction that will send /selector/ to /target/.
  * /data/ will be passed as an additional argument to the method.
- * Note that /data/ is not retained by default.
+ * Note that /data/ is not retained by default (if it is an object).
  */
-- initWithTarget:_target selector:(SEL) _selector data:(const void *) _data
+- (id) initWithTarget:(id) _target selector:(SEL) _selector
+       data:(const void *) _data
 {
     [[super init] setTarget:_target selector:_selector];
     [self setData:_data];
@@ -85,9 +87,9 @@
 
 /*
  * Set the receiver's target and/or selector. If /target/ is |nil| or
- * /selector/ is |NULL|, the previous value is not changed.
+ * /selector/ is |0|, the previous target or selector value is not changed.
  */
-- (void) setTarget:_target selector:(SEL) _selector
+- (void) setTarget:(id) _target selector:(SEL) _selector
 {
     if (_target || _selector == 0) target = _target;
     if (_selector) selector = _selector;
@@ -112,7 +114,7 @@
 
 /*
  * Tell the receiver to retain its user data. Use this only if the data
- * really is some kind of NSObject.
+ * really is some kind of NSObject (or |nil|).
  */
 - (void) retainData
 {
@@ -122,7 +124,7 @@
 /*
  * Return the current target of this action.
  */
-- target
+- (id) target
 {
     return target;
 }
@@ -140,7 +142,7 @@
  */
 - (void *) data
 {
-    return data;
+    return retained ? [[(id) data retain] autorelease] : data;
 }
 
 /*
@@ -157,19 +159,30 @@
  * /args/ and the user data (if it has been set) to the method. The return
  * value will be stored in args[count], which must contain the correct type
  * information for the return value.<p>
- * If the selector's return type is |void| and /count/ is |0|,
- * args may be |NULL|.
+ * If the selector's return type is |void| and /count/ is |0|, args may be
+ * |NULL|.<p>
+ * Note:!! The current implementation will only pass up to 6 arguments from
+ * the /args/ array. If it contains more values, the argument list will be
+ * truncated.
  * @see GTK#-connectSignal:withAction: in class @GTK
  */
-- (void) performWithSender:sender args:(GtkArg *) args count:(unsigned) count
+- (void) performWithSender:(id) sender args:(GtkArg *) args
+	 count:(unsigned) count
 {
-#define MAXARGS	3
+#define MAXARGS	6
     int (*func)(id, SEL, ...) = (int (*)(id, SEL, ...)) method;
-    int is_int[MAXARGS];	// is this type "compatible" to int?
-    int num[MAXARGS];		// integer value of the argument
-    void *ptr[MAXARGS];		// pointer value of the argument
-    int index;
+    struct
+    {
+	gboolean is_int;	// is this type "compatible" to int?
+	union
+	{
+	    int ival;		// integer value of the argument
+	    void *pval;		// pointer value of the argument
+	} arg;
+    } arg[MAXARGS];		// callback argument vector
+    GtkType type;		// argument/return value type info
     int result;
+    int index;
 
     // this code currently does not use the NSInvocation class because:
     //	 - NSInvocation is implemented only on a few systems (yet)
@@ -178,82 +191,92 @@
     // allow up to MAXARGS arguments of type INT, LONG, FLOAT or POINTER
     for (index = 0; index != MAXARGS; ++index)
     {
-	if (index < count)
+	if (index == count)
 	{
-	    GtkType type = args[index].type;
+	    arg[index].is_int = NO;
+	    arg[index].arg.pval = data;
+	    break;
+	}
 
+	type = args[index].type;
 #ifdef DEBUG
-	    if (type == GTK_TYPE_NONE || type == GTK_TYPE_INVALID)
-		g_warning("cannot pass GTK_TYPE_NONE as argument");
+	if (type == GTK_TYPE_NONE || type == GTK_TYPE_INVALID)
+	    g_warning("cannot pass GTK_TYPE_NONE as argument");
 #endif
 #if SIZEOF_LONG != SIZEOF_INT && SIZEOF_LONG != SIZEOF_VOID_P
-	    if (type == GTK_TYPE_LONG || type == GTK_TYPE_ULONG)
-		g_warning("cannot pass GTK_TYPE_LONG on this machine");
+	if (type == GTK_TYPE_LONG || type == GTK_TYPE_ULONG)
+	    g_warning("cannot pass GTK_TYPE_LONG on this machine");
 #endif
 #if SIZEOF_FLOAT != SIZEOF_INT && SIZEOF_FLOAT != SIZEOF_VOID_P
-	    if (type == GTK_TYPE_FLOAT)
-		g_warning("cannot pass GTK_TYPE_FLOAT on this machine");
+	if (type == GTK_TYPE_FLOAT)
+	    g_warning("cannot pass GTK_TYPE_FLOAT on this machine");
 #endif
-	    is_int[index] = type == GTK_TYPE_INT   || type == GTK_TYPE_UINT  ||
-#if SIZEOF_INT == SIZEOF_LONG
-			    type == GTK_TYPE_LONG  || type == GTK_TYPE_ULONG ||
-#endif
-			    type == GTK_TYPE_BOOL  || type == GTK_TYPE_ENUM  ||
-#if SIZEOF_INT == SIZEOF_FLOAT
-			    type == GTK_TYPE_FLOAT ||
-#endif
-			    type == GTK_TYPE_FLAGS;
-	    num[index] = GTK_VALUE_INT(args[index]);
-	    ptr[index] = gtk_type_is_a(type, GTK_TYPE_OBJECT) ?
-			    GTK_GET_OBJECT(GTK_VALUE_OBJECT(args[index])) :
-			    GTK_VALUE_POINTER(args[index]);
-	}
-	else
+	switch (type)
 	{
-	    is_int[index] = 0, ptr[index] = data;
-	    break;
+	    case GTK_TYPE_INT:   case GTK_TYPE_UINT:  case GTK_TYPE_ENUM:
+#if SIZEOF_INT == SIZEOF_LONG
+	    case GTK_TYPE_LONG:  case GTK_TYPE_ULONG:
+#endif
+	    case GTK_TYPE_BOOL:  case GTK_TYPE_FLAGS:
+#if SIZEOF_INT == SIZEOF_FLOAT
+	    case GTK_TYPE_FLOAT:
+#endif
+		arg[index].is_int = YES;
+		arg[index].arg.ival = GTK_VALUE_INT(args[index]);
+		break;
+	    case GTK_TYPE_STRING:
+		arg[index].is_int = NO;
+		arg[index].arg.pval =
+		    gtoolkit_objc_string(GTK_VALUE_STRING(args[index]));
+		break;
+	    default:
+		arg[index].is_int = NO;
+		arg[index].arg.pval = gtk_type_is_a(type, GTK_TYPE_OBJECT) ?
+		    GTOOLKIT_OBJECT(GTK_VALUE_OBJECT(args[index])) :
+		    GTK_VALUE_POINTER(args[index]);
 	}
     }
 
+#define CALL_0()	  (arg[0].is_int ? CALL_1(          ival) \
+					 : CALL_1(          pval))
+#define CALL_1(a)	  (arg[1].is_int ? CALL_2(a,        ival) \
+					 : CALL_2(a,        pval))
+#define CALL_2(a,b)	  (arg[2].is_int ? CALL_3(a,b,      ival) \
+					 : CALL_3(a,b,      pval))
+#define CALL_3(a,b,c)	  (arg[3].is_int ? CALL_4(a,b,c,    ival) \
+					 : CALL_4(a,b,c,    pval))
+#define CALL_4(a,b,c,d)	  (arg[4].is_int ? CALL_5(a,b,c,d,  ival) \
+					 : CALL_5(a,b,c,d,  pval))
+#define CALL_5(a,b,c,d,e) (arg[5].is_int ? CALL_6(a,b,c,d,e,ival) \
+					 : CALL_6(a,b,c,d,e,pval))
+#define CALL_6(a,b,c,d,e,f) \
+	func(target, selector, sender, arg[0].arg.a, arg[1].arg.b, \
+	     arg[2].arg.c, arg[3].arg.d, arg[4].arg.e, arg[5].arg.f, data)
+
     // method is always called with the maximum number of arguments,
     // this is ok because Objective-C uses the C calling convention
-    result =
-    (is_int[0] ?
-	(is_int[1] ?
-	    (is_int[2] ?
-		func(target, selector, sender, num[0], num[1], num[2], data)
-	     :
-		func(target, selector, sender, num[0], num[1], ptr[2], data)
-	    )
-	 :
-	    (is_int[2] ?
-		func(target, selector, sender, num[0], ptr[1], num[2], data)
-	     :
-		func(target, selector, sender, num[0], ptr[1], ptr[2], data)
-	    )
-	)
-     :
-	(is_int[1] ?
-	    (is_int[2] ?
-		func(target, selector, sender, ptr[0], num[1], num[2], data)
-	     :
-		func(target, selector, sender, ptr[0], num[1], ptr[2], data)
-	    )
-	 :
-	    (is_int[2] ?
-		func(target, selector, sender, ptr[0], ptr[1], num[2], data)
-	     :
-		func(target, selector, sender, ptr[0], ptr[1], ptr[2], data)
-	    )
-	)
-    );
+    if (retained) [[(id) data retain] autorelease];
+#if SIZEOF_INT == SIZEOF_VOID_P
+    result = CALL_6(pval, pval, pval, pval, pval, pval);
+#else
+    result = CALL_0();
+#endif
 
+    // return type may be VOID or INT (or "compatible")
+    type = args ? args[count].type : GTK_TYPE_NONE;
 #ifdef DEBUG
-    if (args && args[count].type == GTK_TYPE_INVALID)
+    if (type == GTK_TYPE_INVALID)
 	g_warning("cannot return GTK_TYPE_INVALID");
 #endif
-    // return type may be VOID or INT (or "compatible")
-    if (args && args[count].type != GTK_TYPE_NONE)
+#if SIZEOF_LONG != SIZEOF_INT
+    if (type == GTK_TYPE_LONG || type == GTK_TYPE_ULONG)
+	g_warning("cannot return GTK_TYPE_LONG on this machine");
+#endif
+#if SIZEOF_FLOAT != SIZEOF_INT
+    if (type == GTK_TYPE_FLOAT)
+	g_warning("cannot return GTK_TYPE_FLOAT on this machine");
+#endif
+    if (type != GTK_TYPE_NONE)
 	*GTK_RETLOC_INT(args[count]) = result;
 #undef MAXARGS
 }
