@@ -1,20 +1,20 @@
 /*
  * GToolKit - Objective-C interface to the GIMP Toolkit
- * Copyright (c) 2000  Elmar Ludwig - Universitaet Osnabrueck
+ * Copyright (c) 1998, 1999, 2000  Elmar Ludwig
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  * Boston, MA 02111-1307, USA.
  */
 #ifdef HAVE_CONFIG_H
@@ -23,85 +23,95 @@
 
 #include <stdarg.h>
 #include <stdio.h>
-#include <stdlib.h>
 #include <Foundation/NSArray.h>
 #include <Foundation/NSAutoreleasePool.h>
 #include <Foundation/NSException.h>
 #include <Foundation/NSLock.h>
+#include <Foundation/NSNotification.h>
+#include <Foundation/NSProcessInfo.h>
 #include <Foundation/NSThread.h>
 #include <Foundation/NSUtilities.h>
-#include <GToolKit/GToolKit.h>
+#include <GToolKit/GTKApplication.h>
+#include <GToolKit/GTKButton.h>
+#include <GToolKit/GTKDialog.h>
+#include <gtk/gtkbox.h>
+#include <gtk/gtklabel.h>
+#include <gtk/gtksignal.h>
 
-#if !defined HAVE_NSTHREAD_NOTIFICATION && !defined LIB_FOUNDATION_LIBRARY
-#define NSWillBecomeMultiThreadedNotification	NSBecomingMultiThreaded
-#define NSThreadWillExitNotification		NSThreadExiting
-#endif
-
-static void clicked (GtkWidget *widget)
+/*
+ * Internal functions used by GTKRunMessageBox().
+ */
+static void msgbox_clicked (GtkWidget *widget)
 {
-    [GTKApp stopModalWithCode:[GTK_GET_OBJECT(widget) tag]];
+    [GTKApp stopModalWithCode:[GTOOLKIT_OBJECT(widget) tag]];
 }
 
-static gint delete (GtkWidget *widget, GdkEvent *event)
+static gint msgbox_delete (GtkWidget *widget, GdkEvent *event)
 {
-    clicked(widget);
+    msgbox_clicked(widget);
     return YES;
 }
 
-static GTKButton *insert (GTKBox *box, NSString *label, int tag)
+static GtkWidget *msgbox_insert (GtkWidget *box, NSString *label, int tag)
 {
     GTKButton *button = [GTKButton buttonWithLabel:label];
     GtkButton *gtk_button = [button gtk];
 
     [button setTag:tag];
-    [box packStart:button expand:YES fill:NO padding:0];
+    gtk_box_pack_start((GtkBox *) box, (GtkWidget *) gtk_button, YES, NO, 0);
     gtk_misc_set_padding((GtkMisc *) gtk_button->child, 8, 0);
     GTK_WIDGET_SET_FLAGS(gtk_button, GTK_CAN_DEFAULT);
     gtk_signal_connect((GtkObject *) gtk_button, "clicked",
-		       (GtkSignalFunc) clicked, NULL);
-    return button;
+		       (GtkSignalFunc) msgbox_clicked, NULL);
+    return (GtkWidget *) gtk_button;
 }
 
 /*
- * Display a modal message box or alert panel.
- * Return value is n-1 if button number n is clicked (e.g. 0 for button1).
+ * Display a modal message box or alert panel with up to 3 buttons and
+ * a format string. Return value is n-1 if button number n is clicked.
  */
 int GTKRunMessageBox (NSString *title, NSString *format, NSString *button1,
 		      NSString *button2, NSString *button3, ...)
 {
     GTKDialog *window = [GTKDialog dialog];
-    GTKBox *hbox = (GTKBox *) [window actionArea];	// GTKHButtonBox
-    GTKBox *vbox = (GTKBox *) [window vbox];
-    GTKButton *focus = nil;
-    GTKLabel *label;
+    GtkDialog *gtk_window = [window gtk];
+    GtkWidget *hbox = gtk_window->action_area;	// use GtkHButtonBox here
+    GtkWidget *vbox = gtk_window->vbox;
+    GtkWidget *focus = NULL;
+    GtkWidget *label;
     NSString *text;
     int result, tag = 0;
     va_list ap;
 
-    if (button1) focus = insert(hbox, button1, tag);
-    if (button2) insert(hbox, button2, ++tag);
-    if (button3) insert(hbox, button3, ++tag);
-    if (focus) [focus grabDefault], [focus grabFocus];
-    if (title) [window setTitle:title];
+    if (button1) focus = msgbox_insert(hbox, button1, tag);
+    if (button2) msgbox_insert(hbox, button2, ++tag);
+    if (button3) msgbox_insert(hbox, button3, ++tag);
+    if (focus) gtk_widget_grab_default(focus), gtk_widget_grab_focus(focus);
+    if (title) gtk_window_set_title((GtkWindow *) gtk_window,
+				    gtoolkit_utf8_string(title));
 
     va_start(ap, button3);
     text = [[NSString alloc] initWithFormat:format arguments:ap];
-    label = [GTKLabel labelWithStr:text];
+    label = gtk_label_new(gtoolkit_utf8_string(text));
     [text release];
     va_end(ap);
 
-    [label setPadding:10 ypad:10];
-    [label setJustify:GTK_JUSTIFY_LEFT];
-    [vbox packStartDefaults:label];
     [window setTag:tag];
+    gtk_widget_show(label);
+    gtk_misc_set_padding((GtkMisc *) label, 10, 10);
+    gtk_label_set_justify((GtkLabel *) label, GTK_JUSTIFY_LEFT);
+    gtk_box_pack_start_defaults((GtkBox *) vbox, label);
+    gtk_window_set_position((GtkWindow *) gtk_window, GTK_WIN_POS_CENTER);
 
-    gtk_signal_connect([window gtk], "delete_event", (GtkSignalFunc) delete, 0);
+    gtk_signal_connect((GtkObject *) gtk_window, "delete_event",
+		       (GtkSignalFunc) msgbox_delete, NULL);
     result = [GTKApp runModalForWindow:window];
-    [window destroy];
+    gtk_widget_destroy((GtkWidget *) gtk_window);
     return result;
 }
 
-BOOL gtoolkit_debug;			// enable limited debug output
+extern void gtoolkit_init (void);	// initialize data structures
+
 static NSLock *gtkapp_lock;		// lock for GTKApp instance variables
 
 @implementation GTKApplication
@@ -121,14 +131,14 @@ GTKApplication *GTKApp;			// the application object
 
 /*
  * Return the shared application object (as stored in the variable GTKApp).
- * This method will generate an error message if the application object
+ * This method will raise an NSGenericException if the application object
  * has not yet been created.
  * @see -initWithArgc:argv:
  */
 + (GTKApplication *) sharedApplication
 {
     if (GTKApp == nil)
-	g_error("[GTKApplication sharedApplication]: no application object");
+	[NSException raise:NSGenericException format:@"no application object"];
     return GTKApp;
 }
 
@@ -141,64 +151,65 @@ GTKApplication *GTKApp;			// the application object
     [GTK taskNowMultiThreaded:event];
 }
 
- - init
+ - (id) init
 {
-    g_error("[GTKApp init]: need argc and argv");
+    [NSException raise:NSInvalidArgumentException
+	format:@"[GTKApp init]: need argc and argv"];
     return nil;
 }
 
  - (void) windowDidBecomeMain:(NSNotification *) event
 {
     GTKWindow *window = [event object];
+    BOOL notify = NO;
 
 #ifdef DEBUG
     fprintf(stderr, "[GTKApp windowDidBecomeMain] window = %p\n", window);
 #endif
     if (window == nil) return;
-    if (mainWindow == nil)
+    if (mainWindow == nil) notify = YES;
+    mainWindow = window;
+    if (notify)
 	[center postNotificationName:GTKApplicationDidBecomeActiveNotification
 		object:self];
-    [gtkapp_lock lock];
-    mainWindow = window;
-    [gtkapp_lock unlock];
 }
 
  - (void) windowDidResignMain:(NSNotification *) event
 {
     GTKWindow *window = [event object];
+    BOOL notify = NO;
 
 #ifdef DEBUG
     fprintf(stderr, "[GTKApp windowDidResignMain] window = %p\n", window);
 #endif
     if (window == nil) return;
-    if (mainWindow == window)
-    {
+    if (mainWindow == window) notify = YES, mainWindow = nil;
+    if (notify)
 	[center postNotificationName:GTKApplicationDidResignActiveNotification
 		object:self];
-	[gtkapp_lock lock];
-	mainWindow = nil;
-	[gtkapp_lock unlock];
-    }
 }
 
  - (void) windowWillClose:(NSNotification *) event
 {
     GTKWindow *window = [event object];
     unsigned int count;
+    id _delegate;
 
 #ifdef DEBUG
     fprintf(stderr, "[GTKApp windowWillClose] window = %p\n", window);
 #endif
     if (window == nil) return;
     if (window == mainWindow) [self windowDidResignMain:event];
+
     [gtkapp_lock lock];
     [windows removeObject:window];
     count = [windows count];
+    _delegate = delegate;
     [gtkapp_lock unlock];
 
-    if (count == 0 && (![delegate respondsToSelector:
-	@selector(applicationShouldTerminateAfterLastWindowClosed:)] ||
-	[delegate applicationShouldTerminateAfterLastWindowClosed:self]))
+    if (count == 0 && (![_delegate respondsToSelector:
+	    @selector(applicationShouldTerminateAfterLastWindowClosed:)] ||
+	 [_delegate applicationShouldTerminateAfterLastWindowClosed:self]))
 	[self terminate:self];
 }
 
@@ -209,9 +220,12 @@ GTKApplication *GTKApp;			// the application object
 #ifdef DEBUG
     fprintf(stderr, "[GTKApp windowWillOpen] window = %p\n", window);
 #endif
-    [gtkapp_lock lock];
-    if (window) [windows addObject:window];
-    [gtkapp_lock unlock];
+    if (window)
+    {
+	[gtkapp_lock lock];
+	[windows addObject:window];
+	[gtkapp_lock unlock];
+    }
 }
 
 /*
@@ -225,13 +239,18 @@ GTKApplication *GTKApp;			// the application object
  * GTKApplication class will activate special debug code to trace
  * object allocation and deallocation calls.
  */
-- initWithArgc:(int *) argc argv:(char ***) argv
+- (id) initWithArgc:(int *) argc argv:(char ***) argv
 {
     if (GTKApp == nil)
     {
+#ifdef HAVE_ENVIRON
+	extern char **environ;		// get process environment
+#else
+#define environ	(*argv + *argc)		// (fake) empty environment
+#endif
+
 #ifdef DEBUG
 	fprintf(stderr, "[GTKApp initWithArgc:argv:]\n");
-	gtoolkit_debug = YES;
 #endif
 	GTKApp = [super init];
 	center = [NSNotificationCenter defaultCenter];
@@ -239,8 +258,12 @@ GTKApplication *GTKApp;			// the application object
 
 	gtk_set_locale();
 	gtk_init(argc, argv);
-
-	if (getenv("GTOOLKIT_DEBUG")) gtoolkit_debug = YES;
+	gtoolkit_init();
+	// TODO: this still does not work correctly on GNUstep systems
+#if defined (LIB_FOUNDATION_LIBRARY) || defined (GNUSTEP_BASE_VERSION)
+	[NSProcessInfo initializeWithArguments:*argv count:*argc
+		       environment:environ];
+#endif
 
 	if ([NSThread isMultiThreaded])
 	    [GTKApplication taskNowMultiThreaded:nil];
@@ -279,6 +302,11 @@ GTKApplication *GTKApp;			// the application object
  */
 - (void) finishLaunching
 {
+    NSAutoreleasePool *pool = [NSAutoreleasePool new];
+
+#ifdef DEBUG
+    fprintf(stderr, "[GTKApp finishLaunching]\n");
+#endif
     [center postNotificationName:GTKApplicationWillFinishLaunchingNotification
 	    object:self];
 
@@ -286,6 +314,7 @@ GTKApplication *GTKApp;			// the application object
 
     [center postNotificationName:GTKApplicationDidFinishLaunchingNotification
 	    object:self];
+    [pool release];
 }
 
 /*
@@ -294,30 +323,30 @@ GTKApplication *GTKApp;			// the application object
  * the protocol @proto(GTKApplicationDelegate), it will be notified by
  * the application object on the corresponding events.
  */
-- (void) setDelegate:_delegate
+- (void) setDelegate:(id) object
 {
     [gtkapp_lock lock];
     if (delegate) [center removeObserver:delegate name:nil object:self];
-    delegate = _delegate;
+    delegate = object;
 
-    if ([delegate respondsToSelector:@selector(applicationDidBecomeActive:)])
-	[center addObserver:delegate
+    if ([object respondsToSelector:@selector(applicationDidBecomeActive:)])
+	[center addObserver:object
 		selector:@selector(applicationDidBecomeActive:)
 		name:GTKApplicationDidBecomeActiveNotification object:self];
-    if ([delegate respondsToSelector:@selector(applicationDidFinishLaunching:)])
-	[center addObserver:delegate
+    if ([object respondsToSelector:@selector(applicationDidFinishLaunching:)])
+	[center addObserver:object
 		selector:@selector(applicationDidFinishLaunching:)
 		name:GTKApplicationDidFinishLaunchingNotification object:self];
-    if ([delegate respondsToSelector:@selector(applicationDidResignActive:)])
-	[center addObserver:delegate
+    if ([object respondsToSelector:@selector(applicationDidResignActive:)])
+	[center addObserver:object
 		selector:@selector(applicationDidResignActive:)
 		name:GTKApplicationDidResignActiveNotification object:self];
-    if ([delegate respondsToSelector:@selector(applicationWillFinishLaunching:)])
-	[center addObserver:delegate
+    if ([object respondsToSelector:@selector(applicationWillFinishLaunching:)])
+	[center addObserver:object
 		selector:@selector(applicationWillFinishLaunching:)
 		name:GTKApplicationWillFinishLaunchingNotification object:self];
-    if ([delegate respondsToSelector:@selector(applicationWillTerminate:)])
-	[center addObserver:delegate
+    if ([object respondsToSelector:@selector(applicationWillTerminate:)])
+	[center addObserver:object
 		selector:@selector(applicationWillTerminate:)
 		name:GTKApplicationWillTerminateNotification object:self];
     [gtkapp_lock unlock];
@@ -326,7 +355,7 @@ GTKApplication *GTKApp;			// the application object
 /*
  * Return the application object's delegate.
  */
-- delegate
+- (id) delegate
 {
     return delegate;
 }
@@ -336,8 +365,11 @@ GTKApplication *GTKApp;			// the application object
  */
 - (void) run
 {
+    // should the main loop catch exceptions (like NSApplication)?
+    // it cannot be restartet if it was terminated by an exception
     [self finishLaunching];
     [self main];
+
     if ([NSThread isMultiThreaded]) [self exit:0];	// terminate safely
 }
 
@@ -388,20 +420,18 @@ GTKApplication *GTKApp;			// the application object
  */
 - (void) stopModalWithCode:(int) code
 {
-    [gtkapp_lock lock];
     modalCode = code;
-    [gtkapp_lock unlock];
     [self stop:self];
 }
 
 /*
  * Stop the application's main event loop (if it is running).
  * This will cause the @-run method to return (similar to @-terminate:),
- * but will not send any notifications. If a modal even loop is running,
+ * but will not send any notifications. If a modal event loop is running,
  * it will be stopped instead of the main event loop.
  * The /sender/ parameter is ignored.
  */
-- (void) stop:sender
+- (void) stop:(id) sender
 {
     if ([self isRunning]) [self mainQuit];
 }
@@ -409,7 +439,8 @@ GTKApplication *GTKApp;			// the application object
 /*
  * Tell the application's main event loop to finish (if it is running).
  * This will cause the @-run method to return, which will normally end the
- * program. The /sender/ parameter is ignored.<p>
+ * program. If a modal event loop is running, the program will be terminated
+ * anyway. The /sender/ parameter is ignored.<p>
  * If the delegate implements the method:
  * <pre>
  * - (BOOL) applicationShouldTerminate:(GTKApplication *) sender</pre>
@@ -419,24 +450,28 @@ GTKApplication *GTKApp;			// the application object
  * the GTKApplicationWillTerminateNotification to the default notification
  * center and terminate the event loop.
  */
-- (void) terminate:sender
+- (void) terminate:(id) sender
 {
     static BOOL terminated;
+    volatile BOOL terminate = YES;	// avoid gcc warning
 
     [gtkapp_lock lock];
 #ifdef DEBUG
     fprintf(stderr, "[GTKApp terminate] sender = %p\n", sender);
 #endif
-    if (terminated ||
-	([delegate respondsToSelector:@selector(applicationShouldTerminate:)] &&
-	 [delegate applicationShouldTerminate:self] == NO))
-    {
+    NS_DURING
+	if (terminated || ([delegate respondsToSelector:
+		@selector(applicationShouldTerminate:)] &&
+	     [delegate applicationShouldTerminate:self] == NO))
+	    terminate = NO;
+    NS_HANDLER
 	[gtkapp_lock unlock];
-	return;
-    }
+	[localException raise];
+    NS_ENDHANDLER
 
-    terminated = YES;
+    if (terminate) terminated = YES;
     [gtkapp_lock unlock];
+    if (terminate == NO) return;
 
     [center postNotificationName:GTKApplicationWillTerminateNotification
 	    object:self];
@@ -454,7 +489,7 @@ GTKApplication *GTKApp;			// the application object
     NSString *reason = [exception reason];
 
     if (reason) NSLog(@"%@: %@", [exception name], reason);
-    else NSLog([exception name]);
+    else NSLog([exception name]);	// name must not contain '%'
 }
 
 /*
